# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
import yaml


def load_yaml_file(path):
    """ Returns the parsed structure from a yaml config file.

    :param path: Path where the yaml file is located.
    :return: The yaml configuration represented by the yaml file.
    """
    result = None
    with open(path, 'r') as steam:
        result = yaml.safe_load(steam)
    return result


def get_from_module(module, reference_name):
    """ Return a reference from a module and a class name parameters.
    The reference name must exists in the and could be a variable, a callable
    or a class.

    If reference name doesn't exists in the module it will return None.

    Example:
    >>> get_from_module("my.module", "my_object")
    >>> get_from_module("my.module", "my_function")
    >>> get_from_module("my.module", "MyClass")

    :param basestring module: The module name.
    :param basestring reference_name: What should be returned from the module.
    :return: The value resolved by the module and class name provided or None.
    """
    module = importlib.import_module(module)
    try:
        return getattr(module, reference_name)
    except AttributeError:
        return None


def get_from_string(full_reference):
    """ Return a reference from a string provided as a parameter.
    The reference name must have the an existing module with something to be
    returned from this module. This could be a variable, a callable or a class.

    Example:
    >>> get_from_string("my.module.my_object")
    >>> get_from_string("my.module.my_function")
    >>> get_from_string("my.module.Myclass")

    :param basestring full_reference: Absolute reference.
    :return: The reference resolved from the .
    """
    module = ".".join(full_reference.split(".")[:-1])
    reference = full_reference.split(".")[-1]
    if module.strip() != "":
        return get_from_module(module, reference)
    return None



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def pad(text, **kwargs):
    """ Return a padded text filled with a given character based on a given
    orientation.
    The padded text is sized by default as 79 characters to default as a valid
    pep8 line of code. Add a size parameter will change the padded text size.

    :param text: The text to be padded.
    :param fill: Character to be used to pad the text.
    :param orientation: Orientation to place the fill character. Default is
    'left' but it can be also 'center' and 'right'.
    :param size: Size to be used to pad the text. By default is 79.
    :return: A padded text. """
    size = kwargs.get("size", 79)
    fill = kwargs.get("fill", " ")
    orientation = kwargs.get("orientation", "left").lower()
    align = "<"
    if orientation == "center":
        align = "^"
    elif orientation == "right":
        align = ">"
    pad_format = "".join(["{0:{fill}{align}", str(size), "}"])
    return pad_format.format(text, fill=fill, align=align)


def columnize(text, **kwargs):
    """ Return a text broken in columns. The default columns size is 79 to be a
    valid pep8 line of code. Add the columns parameter to change this value.
    This function will analyze the text based on a category parameter.
    When catetory value is 'sequence' the text will be broken equally in lines
    with same columns(character length) size and last line could be smaller
    than columns value.
    When category value is 'phrase' or 'phrases' the function break lines
    observing words(sequence of characters separated by space). In this case
    the returned text could be padded by an orientation that could be 'left' or
    'center' or 'right' and filled by the fill character.
    The broken text is separated by a newline character.

    :param text: The one line text to break in columns.
    :param category: Category to decide the strategy to break the text in
    columns. Can be 'sequence' or 'phrase' or 'phrases'. Default is 'sequence'.
    :param fill: Character to be used to pad the text.
    :param orientation: Orientation to place the fill character. Default is
    None but it can be also 'left' or 'center' or 'right'.
    :param columns: Value to be used as size to break the text in columns. By
    default is 79.
    :param newline: Character to be used to break to a new line. Default is \n
    :return: A text broken in columns. """

    columns = kwargs.get("columns", 79)
    category = kwargs.get("category", "sequence").lower()
    newline = kwargs.get("newline", "\n")
    orientation = kwargs.get("orientation", None)
    fill = kwargs.get("fill", " ")
    text = text.replace("\n", "")

    if len(text) >= columns:
        broken_text = ""
        if category == "sequence":
            while len(text) > 0:
                if broken_text == "":
                    broken_text = text[0: columns]
                else:
                    chunk = None
                    if len(text) >= columns:
                        chunk = text[0: columns]
                    else:
                        chunk = text[0:]
                    broken_text = "{}{}{}".format(broken_text, newline, chunk)
                text = text[columns:]
        elif category == "phrase" or "phrases":
            text_x = text.split(" ")
            current_line = ""
            while len(text_x) > 0:
                if (len(current_line) + len(text_x[0])) < columns:
                    if current_line == "":
                        current_line = text_x.pop(0)
                    else:
                        current_line = "{} {}".format(
                            current_line, text_x.pop(0)
                        )
                else:
                    if orientation is not None:
                        current_line = pad(
                            current_line,
                            fill=fill,
                            orientation=orientation,
                            size=columns
                        )
                    if broken_text == "":
                        broken_text = current_line
                        current_line = ""
                    else:
                        broken_text = "{}{}{}".format(broken_text, newline,
                                                      current_line)
                        current_line = ""
        return broken_text
    return text



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os


def create_module(module, target):
    """ Create a module directory structure into the target directory. """
    module_x = module.split('.')
    cur_path = ''
    for path in module_x:
        cur_path = os.path.join(cur_path, path)
        if not os.path.isdir(os.path.join(target, cur_path)):
            os.mkdir(os.path.join(target, cur_path))
        if not os.path.exists(os.path.join(target, cur_path, '__init__.py')):
            touch(os.path.join(target, cur_path, '__init__.py'))
    return cur_path


def file_has_extension(filename):
    """ Return True if the informed filename was extension on it.

    :param filename: The filename.
    :return: True if has extension.
    """
    if get_file_extension(filename) is None:
        return False
    return True


def get_file_extension(filename):
    """ Return the extension if the filename has it. None if not.

    :param filename: The filename.
    :return: Extension or None.
    """
    filename_x = filename.split('.')
    if len(filename_x) > 1:
        if filename_x[-1].strip() is not '':
            return filename_x[-1]
    return None


def write(path, data, binary=False):
    """ Write data to a file located in a given path. If binary is true will
    open the file with the binary flag and data should be bytes instead of
    string.
    :param str path: Path where the file is located
    :param str|bytes data: Data to be writen in the file. If binary is true
    data must be in bytes instead of string.
    :param bool binary: If true will read the file with the binary flag
    """
    mode = "w"
    if binary:
        mode = "wb"
    with open(path, mode) as f:
        f.write(data)


def read(path, binary=False):
    """
    Read a file located at the given path. If binary is true will return bytes
    instead of string.

    :param str path: Path where the file is located
    :param bool binary: If true will read the file with the binary flag
    :return str|bytes: File content string or bytes. """
    data = None
    mode = "r"
    if binary:
        mode = "rb"
    abs_path = path
    if not os.path.isabs(abs_path):
        abs_path = os.path.join(os.getcwd(), abs_path)
    with open(abs_path, mode) as f:
        data = f.read()
    return data


def touch(path):
    """ Creates a file located at the given path. """
    with open(path, 'a') as f:
        os.utime(path, None)


def remove_existing(file_path):
    """ Remove a file in a path if it exists and returns true. If file doesn't
    exists returns false.

    :param file_path: The file path
    :return bool: True if file exits
    """
    if os.path.exists(file_path):
        os.remove(file_path)
        return True
    return False


def rmdir_existing(dir_path):
    """ Remove a directory in a path if it exists and returns true. If
    directory doesn't exists returns false.

    :param dir_path: The directory path
    :return bool: True if directory exits
    """
    if os.path.exists(dir_path):
        os.rmdir(dir_path)
        return True
    return False



#!/usr/bin/env python
#
# Copyright 2015-2019 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import platform
from socket import AF_INET, SOCK_STREAM, socket
import subprocess


logger = logging.getLogger(__name__)

TEST_ROOT = os.path.abspath(os.path.dirname(__file__))
FIXTURES_ROOT = os.path.join(TEST_ROOT, "fixtures")
PROJECT_ROOT = os.path.abspath(os.path.join(TEST_ROOT, ".."))


def is_host_up(host, port):
    """
    See: https://bit.ly/2BgCpHI
    :param host:
    :param port:
    :return:
    """
    s = socket(AF_INET, SOCK_STREAM)
    result = s.connect_ex((host, int(port)))
    if result == 0:
        s.close()
        return True
    elif result == 10035:
        logger.warning("Timeout reached")
    else:
        logger.warning("Connection closed")
    s.close()
    return False


def ping(host):
    """ Returns True if host (str) responds to a ping request.
    Remember that a host may not respond to a ping (ICMP) request even if the
    host name is valid.
    Base on https://bit.ly/2TmgeX2 but with pexpect
    :param str host: A host name or ip
    :return boolean: True if ping is replied correctly
    """
    # Option for the number of packets as a function of
    param = '-n' if platform.system().lower() == 'windows' else '-c'
    # Building the command. Ex: "ping -c 1 example.com"
    command = ['ping', param, '1', host]
    return subprocess.call(command) == 0
